function [ImageData] = readcin(name,numimgs)

%readcin.m 
%__________________________________________________________________________
%Jason Kempenaar
%Sandia National Labs, CA
%
%Date Created: August 18,2006
%Date Modified: March 6,2008 LMP
%__________________________________________________________________________
%Description: This function reads images and camera setup data from a 
%   *.cin video file and outputs the data to a structure.
%
%Usage:This function must be handed the file name and the image range
%   (Matlab can not handle the entire image range for large files).
%   i.e.   Data=readcin('Filename',[a,b]) where a to b is the range of
%   image numbers. Note: [0] gives all images.
%   Do not include file extension. 
%   i.e. Data = readcin('c:\PhantomFiles\CoolMovie',[10,50]);
%Note: This file has been written for black and white cines only. Color
%   Cines are uninterpolated and require additional coding.
%__________________________________________________________________________
infile = sprintf('%s.cin',name);
if ~(exist(infile))
    infile = sprintf('%s.cine',name);
end
if ~(exist(infile))
    fprintf('Problem: %s file does not exist\n',infile)
    return;
end
filename=infile;
disp(filename);
fid = fopen(filename,'r');
if fid<1 display([filename ' could not be found']); ImageData=0; return; end
status = fseek(fid,0,'bof');
%__________________________________________________________________________
%Cine File Header
%__________________________________________________________________________

CineHeader.Type = fread(fid,1,'uint16');                   %Marker, must be "CI" 
CineHeader.HeaderSize=fread(fid,1,'uint16');               %Header size in bytes
CineHeader.Compression=fread(fid,1,'uint16');              %CC_RGB=0 - uncompressed BMP
                                                           %CC_JPEG=1 - JPEG Compressed
                                                           %CC_UNINT=2 - Uninterpolated color image for 
                                                           %             the gbrg/rggb CFA
CineHeader.Version=fread(fid,1,'uint16');                  %upgrades, now 1, supports files > 4GB
CineHeader.FirstMovieImage=fread(fid,1,'int32');           %First recorded image number, relative to 
                                                           %trigger
CineHeader.TotalImageCount=fread(fid,1,'uint32');          %Total count of recorded movie images
CineHeader.FirstImageNumber=fread(fid,1,'int32');          %First image saved to this file
                                                           %(relative to trigger)
CineHeader.ImageCount=fread(fid,1,'uint32');               %count of images saved to this file
CineHeader.OffImageHeader=fread(fid,1,'uint32');           %offset in the file of the
                                                           %BITMAPINFO structure for all images
CineHeader.OffSetup=fread(fid,1,'uint32');                 %offset in file of the SETUP structure
CineHeader.OffImageOffsets=fread(fid,1,'uint32');          %offset in file of an array with position
                                                           %of each annotated image in file
CineHeader.TriggerTime.fractions=double(fread(fid,1,'uint32'))/2^32;    %Trigger time 32.32 in second and fraction of 
CineHeader.TriggerTime.seconds=fread(fid,1,'uint32');      %second from Jan 1 1970 (resolution: cca 1/4 
                                                           %nanosecond)
%__________________________________________________________________________
%Windows Structure for Image Header (BitMapInfo)
%__________________________________________________________________________
BitMapInfo.biSize=fread(fid,1,'uint32');                  %header size (without palette)
BitMapInfo.biWidth=fread(fid,1,'int32');                  %image width (pixels)
BitMapInfo.biHeight=fread(fid,1,'int32');                 %image height (pixels)
BitMapInfo.biPlanes=fread(fid,1,'uint16');                %plane of colors
BitMapInfo.biBitCount=fread(fid,1,'uint16');              %bits per pixel
BitMapInfo.biCompression=fread(fid,1,'uint32');           %=0 means no compression
                                                          %not used when the file is JPEG compressed
BitMapInfo.biSizeImage=fread(fid,1,'uint32');             %the size in bytes of the image
BitMapInfo.biXPelsPerMeter=fread(fid,1,'int32');          %horizontal resolution in pixels per meter
BitMapInfo.biYPelsPerMeter=fread(fid,1,'int32');          %vertical resolution in pixels per meter 
BitMapInfo.biClrUsed=fread(fid,1,'uint32');               %the number of color indexes in the 
                                                          %actually used by the bitmap
BitMapInfo.biClrImportant=fread(fid,1,'uint32');          %the number of color indexes in the color 
                                                          %table considered important
%__________________________________________________________________________                                                          
%Camera Setup Information
%__________________________________________________________________________
CameraSetup.FrameRate16=fread(fid,1,'uint16');            %frame rate in pictures per second
CameraSetup.Shutter16=fread(fid,1,'uint16');              %shutter duration in microseconds
CameraSetup.PostTrigger16=fread(fid,1,'uint16');          %the count of the post trigger frames
CameraSetup.FrameDelay16=fread(fid,1,'uint16');           %frame delay in microseconds (Synch Frame 
                                           				  %mode)
CameraSetup.AspectRatio=fread(fid,1,'uint16');            %aspect ratio (width/height)
CameraSetup.ContrastP=fread(fid,1,'uint16');              %the position of the contrast adjustment
CameraSetup.BrightP=fread(fid,1,'uint16');                %the position of the brightness adjustment
CameraSetup.RotateP=fread(fid,1,'uint8');                 %enable the image rotation (90 degrees)
                                                          %(BOOL)
CameraSetup.TimeAnnotation=fread(fid,1,'uint8');          %source of time information
CameraSetup.TrigCine=fread(fid,1,'uint8');                %triggered cine  (BOOL)
CameraSetup.TrigFrame=fread(fid,1,'uint8');               %Synch imaging  (BOOL)
CameraSetup.ShutterOn=fread(fid,1,'uint8');               %enable the shutter (BOOL)
CameraSetup.DescriptionOld=char(fread(fid,121,'char'));   %event description text
CameraSetup.Mark=fread(fid,1,'uint16');                   %will be  "ST"  -  maker for setup file
CameraSetup.Length=fread(fid,1,'uint16');                 %length of the current version of setup
CameraSetup.Binning=fread(fid,1,'uint16');                %binning factor - reduce horizontal slice 
                                                          %dimension
CameraSetup.BinEnable=fread(fid,1,'uint16');              %enable the acqui of the binary sig from 
                                                          %print port
CameraSetup.BinChannels=fread(fid,1,'int16');             %number of multiplexed bytes read from 
                                            			  %parallel port
                                            			  %multiple of 8
CameraSetup.BinSamples=fread(fid,1,'int8');               %number of samples acquired per image; now:1
CameraSetup.BinName=cellstr(char(transpose(reshape...     %8 binary signals names with max 10
    (fread(fid,88,'char'),[11,8]))));                     %chars/name ended each by a NULL byte
CameraSetup.AnaEnable=fread(fid,1,'int16');               %enable the acqui of the analog signals
CameraSetup.AnaChannels=fread(fid,1,'int16');             %number of analog channels used 
CameraSetup.AnaSamples=fread(fid,1,'int8');               %number of samples acquired per image; now: 1
CameraSetup.AnaBoard=fread(fid,1,'int8');                 %board type 0=none, 1=dsk (DSP system kit)
                                                          %2 = DSP+8 channels ADC (12 bit)
CameraSetup.AnaOffset=fread(fid,8,'int16');               %electronic offset correction, per channel
CameraSetup.AnaGain=fread(fid,8,'float');                 %electronic gain correction, and conversion
                                                          %to real units, per channel
CameraSetup.AnaUnit=cellstr(char(transpose(reshape...     %8 analog signals unit strings with max 5
    (fread(fid,48,'char'),[6,8]))));                      %chars/name ended each by a NULL byte
CameraSetup.AnaName=cellstr(char(transpose(reshape...     %8 analog signals names with max 10
    (fread(fid,88,'char'),[11,8]))));                     %chars/name ended each by a NULL byte                                                          

CameraSetup.lFirstImage=fread(fid,1,'int32');             %range of images for continuous recording    
CameraSetup.dwImageCount=fread(fid,1,'uint32');           
CameraSetup.nQFactor=fread(fid,1,'int16');                %Quality  -  for continuous recording; range 
                                                          %2...255                      
CameraSetup.wCineFileType=fread(fid,1,'uint16');          %Cine file type  -  for continuous recording               
CameraSetup.szCinePath=cellstr(char(transpose(reshape...  %4 paths to save cine files - for continuous
    (fread(fid,260,'char'),[65,4]))));                    %recording            
CameraSetup.bMainsFreq=fread(fid,1,'uint16');             %TRUE = 60Hz USA,  FALSE = 50Hz Europe, 
                                                          %for signal view in DSP 
CameraSetup.bTimeCode=fread(fid,1,'int8');                %Time code (IRIG-B, NASA36, IRIG-A ...         
CameraSetup.bPriority=fread(fid,1,'int8');                %Time code has priority over PPS           
CameraSetup.wLeapSecDY=fread(fid,1,'uint16');             %Next day of year with leap second     
CameraSetup.dDelayTC=fread(fid,1,'double');               %Propagation delay for time code      
CameraSetup.dDelayPPS=fread(fid,1,'double');              %Propagation delay for PPS  
CameraSetup.GenBits=fread(fid,1,'uint16');                %General use bits  
                                                          %Bit 0 = Flip vertical (only for v3, not used in v4)
                                                          %      invert image upside down  -  used to
                                                          %      allow the invert of the 256x256. For example 
                                                          %      channels 8...11 may be directed to the FBM
                                                          %      memory SIMMs 0...3 but the resulting image 
                                                          %      will be upside-down. The effect is inverting
                                                          %      bit 3 of the slice address. 0..7 <=> 8..15
                                                          %Bit 1 = Flip horizontal (only for v3, not used in v4)
                                                          %      mirror the image, left-right
                                                          %      added for the color camera where certain components 
                                                          %      are mirrored 
                                                          %      Effect: mirror every row during transfer from FBM to 
                                                          %      memory buffer.
                                                          %Bit 2 = Separate channels v4 . Move the pixels to produce 
                                                          %      rectangular  areas coming out from the same video channel
                                                          %      The video channels are interlaced based on a 8x2 kernel
                                                          %      in the quarter (512x512) camera
CameraSetup.ContrastR=fread(fid,1,'int16');               %values for adjustment in analog      
CameraSetup.BrightR=fread(fid,1,'int16');                 %part of the RGB color system; reused for 
                                                          %digital corrections on V4
CameraSetup.ContrastG=fread(fid,1,'int16');   
CameraSetup.BrightG=fread(fid,1,'int16');   
CameraSetup.ContrastB=fread(fid,1,'int16');   
CameraSetup.BrightB=fread(fid,1,'int16');   
CameraSetup.ImWidth=fread(fid,1,'uint16');                %image dimensions in v4       
CameraSetup.ImHeight=fread(fid,1,'uint16');    
CameraSetup.EDRShutter16=fread(fid,1,'uint16');           %extended dynamic range exposure (v4)    
CameraSetup.Serial=fread(fid,1,'uint32');                 %camera serial number -  will be stored in 
                                                          %every cine file
CameraSetup.Saturation=fread(fid,1,'int16');              %Color saturation [-100, 100]   
CameraSetup.Reserved=fread(fid,3,'uint8');                %align to dword  
CameraSetup.AutoExp=fread(fid,1,'uint32');                %autoexposure   
CameraSetup.bFlipH=fread(fid,1,'uint32');                 %Flip horizontal, vertical in v4   
CameraSetup.bFlipV=fread(fid,1,'uint32');                 %For color images flips are postponed after
                                                          %interpolation
CameraSetup.bCrossHair=fread(fid,1,'uint32');             %display a crosshair in setup   
CameraSetup.FrameRate=fread(fid,1,'uint32');                
CameraSetup.Shutter=fread(fid,1,'uint32');   
CameraSetup.EDRShutter=fread(fid,1,'uint32');   
CameraSetup.PostTrigger=fread(fid,1,'uint32');   
CameraSetup.FrameDelay=fread(fid,1,'uint32');   
CameraSetup.bEnableColor=fread(fid,1,'uint32');           %available to user: when 0 force gray images    
CameraSetup.CameraVersion=fread(fid,1,'uint32');          %4, 5 ....   
CameraSetup.FirmwareVersion=fread(fid,1,'uint32');        %Firmware version   
CameraSetup.SoftwareVersion=fread(fid,1,'uint32');        %Phantom version   
CameraSetup.RecordingTimeZone=fread(fid,1,'int32');       %the time zone active during the recording of   
                                                          %the cine
CameraSetup.CFA=fread(fid,1,'uint32');                    %code for the color filter array (for late 
                                                          %interpolate or uninterpolate): 
                                                          %CFA_NONE=0,(gray) CFA_VRI=1(gbrg/rggb), 
                                                          %CFA_VRIV6=2(bggr/grbg), CFA_BAYER=3(gb/rg)
                                                          %high byte carry info about color/gray heads at 
                                                          %v6 
                                                          %Masks: 0x80000000: TLgray 0x40000000: TRgray 
                                                          %0x20000000: BLgray 0x10000000: BRgray
                                                          %Final adjustments after image processing:
   
CameraSetup.Bright=fread(fid,1,'int32');                  %Brightness -100...100 neutral:0     
CameraSetup.Contrast=fread(fid,1,'int32');                %Contrast -100...100 neutral:0   
CameraSetup.Gamma=fread(fid,1,'int32');                   %Gamma -100...100 neutral:0   
CameraSetup.Reserved1=fread(fid,1,'uint32');              %BOOL LockToIRIG;%camera mode : lock to IRIG  
                                                          %- removed
CameraSetup.AutoExpLevel=fread(fid,1,'uint32');           %level for autoexposure control   
CameraSetup.AutoExpSpeed=fread(fid,1,'uint32');           %speed for autoexposure control   
CameraSetup.AutoExpRect=fread(fid,4,'uint32');            %rectangle for autoexposure control   
CameraSetup.WBGain=fread(fid,8,'float');                  %Gain adjust on R,B components, for white 
                                                          %balance,
                                                          %1.0 = do nothing, 
                                                          %index 0: all image for v4,5,7...TL head for v6
                                                          %index 1, 2, 3 :   TR, BL, BR for v6
CameraSetup.Rotate=fread(fid,1,'int32');                  %0=do nothing +90=counterclockwise 
                                                          %-90=clockwise
CameraSetup.WBView=fread(fid,2,'float');                  %White balance to apply on images from cine 
                                                          %file
CameraSetup.RealBPP=fread(fid,1,'uint32');                %real number of camera bits per pixel 
                                                          %e.g 8 on old cameras and 12 on v7 with 12 bit 
                                                          %converters; pixel stored on 16 bit
CameraSetup.Conv8Min=fread(fid,1,'uint32');               %Minimum value when convert to 8 bits   
CameraSetup.Conv8Max=fread(fid,1,'uint32');               %Max value when convert to 8 bits 
CameraSetup.FilterCode=fread(fid,1,'int32');              %ImageProcessing: area processing code

%--NOTE--: Values on Remaining CameraSetup have not been verified
  %See phint.h for more info and variable
CameraSetup.FilterParam=fread(fid,4,'int32');   
CameraSetup.UF=fread(fid,25,'int32');                      %user filter, see PhInt.h   
CameraSetup.BlackCalSVer=fread(fid,1,'uint32');           %Black Calibration SoftwareVersion   
CameraSetup.WhiteCalSVer=fread(fid,1,'uint32');           %White Calibration SoftwareVersion   
CameraSetup.GrayCalSVer=fread(fid,1,'uint32');            %Gray Calibration SoftwareVersion   
CameraSetup.bStampTime=fread(fid,1,'uint32');             %Stamp time (in continuous recording)
                                                          %1 = absolute time, 3 = from trigger
CameraSetup.SoundDest=fread(fid,1,'uint32');              %0: none, 1:Speaker, 2: sound board
CameraSetup.FRPSteps=fread(fid,1,'uint32');               %Suplimentary steps in frame rate profile
                                                          %0 means no frame rate profile
CameraSetup.FRPImgNr=fread(fid,16,'int32');               %Image number where to change the rate and/or exposure
                                                          %allocated for 16 points (4 available in v7)
CameraSetup.FRPRate=fread(fid,16,'uint32');               %new value for frame rate (fps)
CameraSetup.FRPExp=fread(fid,16,'uint32');                %new value for exposure (nanoseconds)
CameraSetup.MCCnt=fread(fid,1,'int32');                   %Partition count (cine count - 1) 
                                                          %Preview cine does not need a partition 
CameraSetup.MCPercent=fread(fid,64,'float');              %Percent of memory used for partitions 
                                                          %allocate for 64 partitions, use 15 initially 
CameraSetup.CICalib=fread(fid,1,'uint32');                %This cine or this stg is the result of
                                                          %a current image calibration 
                                                          %mask: 1 BlackRef, 2 WhiteCalib, 4 GrayCheck
%Last cicalib done at the acqui params:                                                          
CameraSetup.CalibWidth=fread(fid,1,'uint32');             %Image dimensions  
CameraSetup.CalibHeight=fread(fid,1,'uint32');              
CameraSetup.CalibRate=fread(fid,1,'uint32');              %Frame rate (frames per second)
CameraSetup.CalibExp=fread(fid,1,'uint32');               %Exposure duration (microseconds)
CameraSetup.CalibEDR=fread(fid,1,'uint32');               %EDR (microseconds)
CameraSetup.CalibTemp=fread(fid,1,'uint32');              %Temperature
CameraSetup.HeadSerial=fread(fid,4,'uint32');             %for ethernet multihead cameras (v6.2)
                                                          %When multiple heads are saved in a file existing head serials are not zero
                                                          %When one head is saved in a file its serial is is in HeadSerial[0] and the other 
                                                          %head serials are 0xFFffFFff
CameraSetup.RangeCode=fread(fid,1,'uint32');              %range data code: describe the range data format 
CameraSetup.RangeSize=fread(fid,1,'uint32');              %range data,  per image size
CameraSetup.Decimation=fread(fid,1,'uint32');             %Reduce the frame rate when sending the images to i3 through fiber 
CameraSetup.MasterSerial=fread(fid,1,'uint32');           %Master camera Serial for external sync. 0 means none
CameraSetup.Sensor=fread(fid,1,'uint32');                 %Camera sensor code 
CameraSetup.ShutterNs=fread(fid,1,'uint32');              %Exposure, nanoseconds
CameraSetup.EDRShutterNs=fread(fid,1,'uint32');           %EDRExp, nanoseconds
CameraSetup.FrameDelayNs=fread(fid,1,'uint32');           %FrameDelay, nanoseconds
CameraSetup.ImPosXAcq=fread(fid,1,'uint32');              %acquired image horizontal offset in sideStamped image
CameraSetup.ImPosYAcq=fread(fid,1,'uint32');              %acquired image vertical offset in sideStamped image
CameraSetup.ImWidthAcq=fread(fid,1,'uint32');             %acquired image width (diff val from ImWidth if sideStamped file)
CameraSetup.ImHeightAcq=fread(fid,1,'uint32');            %acquired image height (diff val from ImWidth if sideStamped file)
c=CineHeader.OffImageOffsets-ftell(fid);
CameraSetup.Description=char(fread(fid,c,'char'));        %length of description=???

%__________________________________________________________________________
%Time Stamp and Exposure Data
%__________________________________________________________________________

b=(CineHeader.OffSetup+CameraSetup.Length);
if b<CineHeader.OffImageOffsets
    status = fseek(fid,b,'bof');
    TimeStamps.BlockSize_time=fread(fid,1,'uint32');    %length of block in bits
    TimeStamps.Type_time=fread(fid,1,'uint16');     %Type=1002 for time block
    TimeStamps.Reserved_time=fread(fid,1,'uint16');
    DataLength=(TimeStamps.BlockSize_time-8);
    for j=1:DataLength/8
        TimeStamps.Data_Fractions(j,1)=double(fread(fid,1,'uint32'))/2^32; %fractions of sec since 1970
        TimeStamps.Data_Seconds(j,1)=fread(fid,1,'uint32'); %seconds since 1970
    end

    TimeStamps.BlockSize_exp=fread(fid,1,'uint32');     %length of block in bits
    TimeStamps.Type_exp=fread(fid,1,'uint16');      %Type=1003 for Exposure Block
    TimeStamps.Reserved_exp=fread(fid,1,'uint16');
    DataLength=(TimeStamps.BlockSize_exp-8);
    TimeStamps.Data_exp=fread(fid,DataLength/4,'uint32')/2^32; %Exposure in secs
   
end
%__________________________________________________________________________
%Read Raw Images
%__________________________________________________________________________

% Initialize variables
Ncol = BitMapInfo.biWidth; % # of columns in a single image
Nrow = BitMapInfo.biHeight;% # of rows in a single image
Nimg =CineHeader.ImageCount; % # of images to be read from the current file
status = fseek(fid,CineHeader.OffImageOffsets,'bof');
Image.pImage=fread(fid,Nimg,'uint64'); % position of images in file
Npel = Ncol.*Nrow; % # of pixels per image
if numimgs==0 %reads all images 
    Nstop=Nimg;
    Nstart=1;
elseif (length(numimgs) == 1) % just read the selected image
    Nstop=numimgs;
    Nstart=numimgs;
    Nimg=1;
else % reads selected range of images
    Nstop=numimgs(1,2);
    Nstart=numimgs(1,1);
    Nimg=Nstop-Nstart+1;
end
A = zeros(Npel.*Nimg,1,'uint16');

% Read the image data into array
% fprintf('  Reading binary image data...');
for i = Nstart:Nstop 
   status = fseek(fid,Image.pImage(i,1),'bof');
   Image.AnnotationSize(i,1)=fread(fid,1,'uint32'); %AnnotationSize
   if (Image.AnnotationSize(i,1)-8)>0 %Read annotation if present
       Image.Annotation(i,1)=fread(fid,(Image.AnnotationSize(i,1)-8),'uint32');
   end
   Image.ImageSize=fread(fid,1,'uint32'); % Image Size in Bytes.
   istart = (i-Nstart).*Npel + 1;
   iend =istart+Npel-1;
   [A(istart:iend),count] = fread(fid,Npel,'uint16'); % Reads Each Image into vector
end;
status = fclose(fid);
% fprintf('done.\n');

%Reshape vector into properly-dimensioned image matrix
N = [Ncol Nrow Nimg];
Image.RawImages = reshape(A,N);


%Output Data Structures
ImageData.CameraSetup=CameraSetup;
ImageData.CineHeader=CineHeader;
ImageData.BitMapInfo=BitMapInfo;
ImageData.Images=Image;
ImageData.TimeStamps=TimeStamps;
