function [x,y]=JBtime_stdev(TestImage1,TestImage2, TestImage3,showfigs, ...
    nhsize,strip,thresval,num_erodes,num_dilates, downstreamStrip, ...
    TopStrip, BottomStrip, XMark, YMark, JetCenter, liqCoreMask, x_c, ...
    complement, mask)

% function calculates the standard deviation of intensity of each image
% pixel over three consecutive frames. The resulting image (high stdev
% where the jet is and low stdev for the background) is then analysed to
% find the jet boundary - Timothy C.Williams 1/23/2008

orgn = [YMark x_c];
nColumns = size(TestImage1,2);
nRows = size(TestImage1,1);

TestImage=zeros(nRows, nColumns,3); % Matrix of three image frames used to calculate standard deviation for each pixel over 3 frames
TestImage(:,:,1) = TestImage1;
TestImage(:,:,2) = TestImage2;
TestImage(:,:,3) = TestImage3;

stdev_3frames = std(TestImage,1,3); %standard deviation of each image pixel in the 'time' of frame dimension 

nh=ones(nhsize,nhsize); %pixel size of region used for standard deviation 
stdv_image = stdfilt(stdev_3frames,nh); %standard deviation across the standard deviation wrt time/frame result from above image

%normalize image by max value and then apply user supplied threshold %%%%%%
maxlevel=max(max(stdv_image));
stdv_image=stdv_image/maxlevel;
level = thresval/maxlevel;
if level > 1; %threshold level can't be greater than 1
    x=[];y=[];
    return; 
end;

BW_stdvIm = im2bw(stdv_image,level) & mask;
%figure;imagesc(BW_stdvIm);

% Complement the BW image if desired
if complement
    BW_stdvIm = imcomplement(BW_stdvIm);
end

BW_stdvIm(:,1:strip)=0; %make strip next to injector 0
BW_stdvIm(:,downstreamStrip:nColumns)=0;  %make downstream strip 0
BW_stdvIm(1:TopStrip,:)=0;  %make top strip 0
BW_stdvIm(nRows-BottomStrip:nRows,:)=0;  %make bottom strip 0

% find the biggest region in the image %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L = bwlabel(BW_stdvIm,8); %Find the already connected regions within the input image IM 

%figure;imagesc(L);
RegionAreas = regionprops(L, 'area'); % if it hasn't then find the largest region
Area = cat(1, RegionAreas.Area);

if Area < 100; %stops the program crashing
    x=[];y=[];return;
end;

LabelNum=find(Area==max(Area));


if isempty(LabelNum) == 1;
    x=[];y=[];return;
end;  
if LabelNum == 0;
    x=[];y=[];return;
end;

JET = zeros(size(BW_stdvIm));
JET(L==LabelNum)=1; %create a image consisting of just the largest region

try
    % split the image into a top half and a bottom half using the user supplied 'JetCenter'
    JET_top = logical(JET(1:JetCenter,:));
    JET_bot = logical(JET(JetCenter+1:nRows,:));
    %figure(4); imagesc(JET_top);
    %figure(5); imagesc(JET_bot);
    %figure(6);
    Extrema_top = regionprops(JET_top, 'Extrema'); %find extremities of top region
    Extrema_bot = regionprops(JET_bot, 'Extrema'); %find extremities of bottom region

    TopRegion = Extrema_top(1,1).Extrema(7,:)+0.5; %convert pixel center locations to actual pixel locations in image
    BotRegion = Extrema_bot(1,1).Extrema(8,:)+0.5;
    BotRegion(1,2) = BotRegion(1,2) + JetCenter; % take account of split image, so can apply coordinates to full image

    %%connect left edges of the top and bottom regions
    if TopRegion(1,1) < BotRegion(1,1) %left edge of top region is to the left of the left edge of the bottom region
        JET(TopRegion(1,2),TopRegion(1,1):BotRegion(1,1)) = 1; %connect horizontally
        JET(TopRegion(1,2):BotRegion(1,2),BotRegion(1,1)) = 1;  %connect vertically  
    elseif BotRegion(1,1) < TopRegion(1,1) %left edge of bottom region is to the left of the left edge of the top region
        JET(BotRegion(1,2),BotRegion(1,1):TopRegion(1,1)) = 1; %connect horizontally
        JET(TopRegion(1,2):BotRegion(1,2),TopRegion(1,1)) = 1; %connect vertically 
    else
        JET(TopRegion(1,2):BotRegion(1,2),TopRegion(1,1)) = 1; %left edges are vertically in line
    end;    
catch % If you split it in half and one of the halves is empty the code fails. Just skip this part.
    
end

JET = imfill(JET,'holes');
JET = bwmorph(JET,'erode',num_erodes); 
JET = bwmorph(JET,'dilate',num_dilates); 
JET = imfill(JET,'holes');

% Find max and min of jet edges and block out liquid core
% liquid core blocked out inside of 1/3 of max edge
if liqCoreMask % Another user option - can be turned off
    [b]=jetBoundary(JET);
    [S,indxmax]=max(b(:,2));                                                   %#ok<NASGU>
    [y1,indy1]=min(b(:,1)); a= b(:,1)==y1; indx1=min(b(a,2));                  %#ok<NASGU>
    [y2,indy2]=max(b(:,1)); a= b(:,1)==y2; indx2=min(b(a,2));                  %#ok<NASGU>
    yblock=max([orgn(1)-y1 y2-orgn(1)])/3; 
    xblock=min([indx1 indx2]);
    %xpoly=[orgn(2)-1 xblock mean([xblock S]) S mean([xblock S]) xblock]; 
    xpoly=[orgn(2)-1 xblock mean([xblock S]) mean([xblock S]) mean([xblock S]) xblock]; % Changed so it's not so pointy at the end of the liquid core mask. RKG 4/28/10
    ypoly=orgn(1)+[0 -yblock -yblock/1.3 0 yblock/1.3 yblock];
    BWmask=roipoly(nRows,nColumns,xpoly,ypoly);
    JET=JET|BWmask; %adds liquid core to jet boundary JET
end

% Create a 'disk' structuring element for an imclose operation:
% seDiskRadius = 2;
% seDiskN      = 4;
% seDisk = strel('disk',seDiskRadius,seDiskN);
% JET=imclose(JET,seDisk);


% erodes can form disconnected regions to form, this part keeps the largest region
L = bwlabel(JET,8);    
RegionAreas = regionprops(L, 'area'); % if it hasn't then find the largest region
Area = cat(1, RegionAreas.Area);
LabelNum=find(Area==max(Area));
JET = zeros(size(JET));
JET(L==LabelNum)=1;

% find the boundary of the region to send back to the GUI as x,y coordinates
JetBoundary = bwboundaries(JET);
x = JetBoundary{1}(:,2);
y = JetBoundary{1}(:,1);

if showfigs==1
end;

