function [time,vapPen,tan2,JB,maxV] = aviVSD(inFile,imgNums,origin,CRng,angleParams,opts) 
% aviVSD computes the jet boundary of a *.cine or *.mat file.

% --------------------------- Outputs -------------------------------------

% time          Array of time values in us
% vapPen        Array of vapor penetration values in mm
% tan2          Array of spray angles (full angle) in degrees
% JB            The computed jet boundary matrix
% maxV          The maximum penetration distance in mm

% --------------------------- Inputs --------------------------------------

% inFile        The name of the cine file or mat file
% outFile       The name of the output avi file
% imgNums       The image numbers to be used in the file
% origin        An array which contains the (x,y) origin
% CRng          An array which contains the [low high] color range, where
%               the number stored in low is displayed as black, the number
%               stored in high is displayed as white and all numbers in
%               between are considered intermediate shades of gray.
% opts          A cell array which includes a number of processing options,
%               detailed below.

% Example calls
%[time,vapPen,tan2,JB,maxV] = aviVSD('C:\xdata\dn\schlieren\bklda085.cine',3:25,[32 130],[0 2500],[0.45 0.55],[3 0 511 0 0 60 1 2 1/5.4 0 8 50 0 1 1 -1 1 23]);

    xOrigin         = origin(1) ; 
    yOrigin         = origin(2);

    avgStartPer     = angleParams(1); % Typically: 0.45
    avgEndPer       = angleParams(2); % Typically: 0.55

    stdFiltSize     = int8(opts{1}) ; % Size of standard deviation filter
    injStrip        = opts{2} ; % Block out this # of pixels to left of injector
    downStreamStrip = opts{3} ; % Block out from this pixel # to the right edge
    topStrip        = opts{4} ; % Block out this # of pixels from the top edge
    bottomStrip     = opts{5} ; % Block out this # of pixels from the bottom edge
    threshVal       = opts{6} ; % Array of Threshold values for im2bw
    numErodes       = opts{7} ; % Sets the number of erodes to use
    numDilates      = opts{8} ; % Sets the number of dilates to use
    scale           = opts{9} ; % Image scale in mm / pixel
    imageFlipLR     = opts{10}; % Flips the image horizontally before processing
    injStartFrame   = opts{11}; % Injection Start Frame for time alignment
    endInjTime      = opts{12}; % End of injection time in ms
    complement      = opts{13}; % Complements the image if boolean = 1
    liqCoreMask     = opts{14}; % Masks a liquid core back to origin if boolean = 1
    useSubFrame     = opts{15}; % Uses a subtraction frame if boolean = 1
    subFrame        = opts{16}; % Subtraction frame used if useSubFrame = 1.
    siebers         = opts{17}; % Use the siebers method for spray angle if boolean = 1. Alternative is Ryan's method.
    thetaInput      = opts{18}; % Input a spray angle if desired rather than having measureSpray actually calculate it.


    % Check the opts inputs and display proper error messages
    if rem(stdFiltSize,2) && stdFiltSize > 0
        % stdFiltSize is a positive odd integer
    else
        fprintf('Error: stdFiltSize must be a positive odd integer!\n')
        time = []; vapPen = []; tan2 = []; JB = []; maxV = []; return
    end

    % Strip the *.cin in preparation for the readcin function
    [p,filename,ext] = fileparts(inFile);                                    %#ok<NASGU,ASGLU> for backwards compatibility
    isCineFile = strcmp(ext,'.cine');
    isCinFile  = strcmp(ext,'.cin');
    isMatFile  = strcmp(ext,'.mat');
    if isCineFile
        cineFilePath = strrep(inFile,'.cine',''); isMatFile = 0;
    elseif isCinFile
        cineFilePath = strrep(inFile,'.cin',''); isMatFile = 0;
    elseif isMatFile % Can handle imported data from IFP (Paris). see ifpImageConvert.m
        load(inFile)
        frameRate16 = double(frameRate16);                                     %#ok<NODEF>
    else
        fprintf('The file %s is not a known file type. Please use *.cine, *.cin, or *.mat files\n',inFile)
        time = []; vapPen = []; tan2 = []; JB = []; maxV = []; return
    end

    if isCineFile || isCinFile
        % Load one image to get header information.
        fprintf('Reading header from %s.\n\n',inFile)
        imdata = readcin(cineFilePath,[1,1]);

        % Get the frame number of the first frame
        firstImageNumber = imdata.CineHeader.FirstImageNumber;                 

        % Get total number of frames in the cine file
        imageCount = imdata.CineHeader.ImageCount;                             

        % Get the frame rate from the camera
        frameRate16 = imdata.CameraSetup.FrameRate16; 
    end    

    % Display Experiment Information from the txtfile name information
%     [O2,Temp,Density,InjPress,Orifice]=dcsfFileExtract(filename);
%     InjPress = InjPress * 0.00689475728; % Conversion to MPa from psi

    % Print the header information
    fprintf('First Image Number: %d\n',firstImageNumber + 1)
    fprintf('Image Count:        %d\n',imageCount)
    fprintf('Frame Rate:         %d frames per second\n',frameRate16)
%     fprintf('Injection Pressure: %f MPa \n',InjPress)
%     fprintf('O2 Percentage:      %d%%\n',O2)
%     fprintf('Temperature:        %d K\n',Temp)
%     fprintf('Density:            %f kg/m^3 \n',Density)
%     fprintf('Orifice Diameter    %f mm\n',Orifice)

    % Check the imgNums input and return if not in range
    if min(imgNums) < firstImageNumber + 1
        fprintf('\nError: The first image number is %d!\n', ...
            firstImageNumber + 1)
        time = []; vapPen = []; tan2 = []; JB = []; maxV = []; return
    end
    if max(imgNums) > firstImageNumber + imageCount
        fprintf('\nError: The last image number is %d!\n', ...
            firstImageNumber + imageCount)
        time = []; vapPen = []; tan2 = []; JB = []; maxV = []; return
    end

    startFrame = imgNums(1);
    totalFrames = imgNums(end)-imgNums(1);

    % Validate frame numbers to be loaded
    if isMatFile
        if totalFrames < 1;                                                    
            totalFrames = 1;
        set(handles.edtTotalFrames, 'String', totalFrames);
        end
        if startFrame > imageCount                                             
            startFrame = imageCount;
        end
        if startFrame + totalFrames - 1 > imageCount
            totalFrames = imageCount - startFrame;
            set(handles.edtTotalFrames, 'String', totalFrames);
        end
        offsetRead = 1;   
        offsetWrite = 0;                                                       
    else
        if startFrame > firstImageNumber + imageCount - 1;
        startFrame = firstImageNumber + imageCount - 1;
        set(handles.edtStartFrame, 'String', startFrame);
        end 
        if startFrame + totalFrames - 1 > firstImageNumber + imageCount - 1;
            totalFrames = firstImageNumber + imageCount - startFrame;
            set(handles.edtTotalFrames, 'String', totalFrames);
        end 
        if startFrame < firstImageNumber;
            startFrame = firstImageNumber;
            set(handles.edtStartFrame, 'String', startFrame);
        end
        % Offset to the frame number used for the cine read program, e.g., the
        % first frame of the cine textfile might be (-57) but it is the first (1)
        % frame in the cine file.
        offsetRead = (firstImageNumber*-1)+1;  

        % Offset to the frame number used for writing frames to the screen, e.g.,
        % the first image in the image array could be frame (-10).
        offsetWrite = (startFrame*-1)+1;                                       
    end

    if isMatFile
        % Images to process:
        endFrame = startFrame + totalFrames -1;
        images = images(:,:,startFrame:endFrame);                              %#ok<NODEF> Would be loaded from *.mat file
    else
        fprintf('\nReading frames %d through %d in %s ...', ...
        min(imgNums),max(imgNums),inFile)
        imdata = readcin90(cineFilePath, ...
            [startFrame + offsetRead,startFrame + totalFrames + offsetRead]);
        images = imdata.Images.RawImages;
        imSize(1) = imdata.CameraSetup.ImWidth;
        imSize(2) = imdata.CameraSetup.ImHeight;                               %#ok<NASGU>
        fprintf(' Done!\n')
    end

    if imageFlipLR
        for i = 1:size(images,3)
            images(:,:,i) = fliplr(images(:,:,i));
        end
    end

    % Mask out large spreading angle near injector LMP 2/4/2009
    % Make mask to block out regions not of interest tan(theta/2) of 0.3 
    % Edited RKG 8/31/09 to reflect actual calculated y-origin (though x not
    % automatic and currently has no box in GUI to change it.
    dimIm = size(images(:,:,1));
    %t2=0.3; % tangent(theta/2) tangent of half angle of mask
    t2=0.4; % wider angle for bklda (RKG 4/28/10)
    xpolym = [xOrigin-10 yOrigin/t2 dimIm(2) dimIm(2) (dimIm(1)-yOrigin)/t2]; 
    ypolym = [yOrigin 0 0 dimIm(1) dimIm(1)];
    mask = roipoly(dimIm(1),dimIm(2),xpolym,ypolym); % Region of interest polygon

    % Get the image time (as related to the actual SOI signal) directly from 
    %   the cine file header. This properly aligns the timebase with the images.
    if isMatFile
        % time1 was loaded in above (in seconds, this is in us)
        time1 = time1*1e6;                                                     %#ok<NODEF> 
        %time2 = time1(currentFrame);                                           %#ok<NASGU>
    else
        time1 = (imdata.TimeStamps.Data_Fractions - ...
              imdata.CineHeader.TriggerTime.fractions)*1e6 - ...
             (imdata.TimeStamps.Data_Fractions(injStartFrame+1) - ...
              imdata.TimeStamps.Data_Fractions(1))*1e6;
        %time2 = time1(currentFrame);                                           %#ok<NASGU>
    end



    % Dislpay images / Compute boundary
    % This is the main loop in which all the magic takes place.

    %close all hidden
    figure(10)
    colormap(gray);
    stepSize = 1;
    origSubFrame = subFrame;
    fprintf('Current Frame =    ')

    % Preallocate arrays
    %totalOutputFrames =  ...
    %    length(length(origCurrentFrame:stepSize:startFrame + totalFrames - 1));
    time   = zeros(1,startFrame+totalFrames+1);
    vapPen = zeros(1,startFrame+totalFrames+1);
    tan2   = zeros(1,startFrame+totalFrames+1);
    JB     = cell(startFrame+totalFrames+1,2);
    maxV   = zeros(1,startFrame+totalFrames+1);
    tcount = 1; %threshold counter
    tlen   = length(threshVal); %threshold array length
    if tlen < totalFrames
        if tlen == 1 %if only 1 threshVal
           threshVal = repmat(threshVal,1,totalFrames); %repeats threshVal for all frames
        else %fill in rest of threshVals with last threshVal in array
           threshVal = [threshVal repmat(threshVal(end),1,totalFrames-tlen)]; 
        end
    end
    for currentFrame = startFrame + 2:stepSize:startFrame + totalFrames
        TestImage1 = images(:,:,currentFrame + offsetWrite);
        TestImage2 = images(:,:,currentFrame + offsetWrite - 1);
        TestImage3 = images(:,:,currentFrame + offsetWrite - 2);
        showfigs = 0; % dummy input for JBtime_stdev. No longer used.
        i = currentFrame;
        % I suppose ultimately any of the JB methods can be used here if 
        % desired. The inputs would just have to be adjusted, and perhaps new
        % ones added.

        [x,y] = JBtime_stdev(TestImage1, TestImage2, TestImage3, ...
            showfigs, stdFiltSize, injStrip, threshVal(tcount), ...
            numErodes, numDilates,  ...
            downStreamStrip, topStrip, bottomStrip, ...
            xOrigin, yOrigin, yOrigin, liqCoreMask, xOrigin, ...
            complement, mask);
        tcount = tcount + 1;
        % Subtraction frames. This part of the code was written by Nathan Greco.
        if useSubFrame
            if subFrame == currentFrame
                subframe = startFrame;                                         %#ok<NASGU>
            end
        if currentFrame == startFrame
            waitfor(msgbox( ... 
            sprintf('Unable to subtract frame.\nCurrent frame is also the starting frame.'),...
            'Error'));
            time = []; vapPen = []; tan2 = []; JB = []; maxV = []; return
        end
        if origSubFrame <= -1
            if currentFrame > startFrame
                subFrame = currentFrame-1;
            else
                waitfor(msgbox( ... 
                sprintf('Unable to subtract previous frame.\nCurrent frame is also the starting frame.'),...
                'Error'));
                time = []; vapPen = []; tan2 = []; JB = []; maxV = []; return
            end
        end
            TestImage1 = TestImage1 - images(:,:,subFrame+offsetWrite) + 2000;
        end

        % Create a blank image to fill with the boundary
        bwimg = false(dimIm);                                         

        % Fill the blank image with the boundary
        for j = 1:size(x)
            bwimg(y(j),x(j)) = 1;                                       
        end

        % Fill boundary
        bwimg = imfill(bwimg,'holes');

        % Calculate a penetration distance and an 'X marks the spot'
        percentAdj = 0.80; % Adjusts the placement of the 'X marks the spot'
                           % relative to the calculated penetration distance.
                           % The 'blob' in which the X lies is considered the
                           % spray.
        time(i) = time1(currentFrame+1);
%         dist = jetPenCalculation(Orifice*1000, ...
%                                  Density,700, ...
%                                  InjPress, ...
%                                  time(i)/1000)/scale * percentAdj;           %#ok<NASGU> % dist into chamber x adjustment factor
        [c_L,S,theta,A_p,theta_l] = measureSpray(bwimg, ...
                                                 xOrigin,yOrigin, ...
                                                 scale, ...
                                                 siebers, ...
                                                 avgStartPer,avgEndPer, ...
                                                 thetaInput);       %#ok<NASGU>
        xval = ([1:dimIm(2)]-xOrigin)*scale;                                   %#ok<NBRAK>
        rad =  ([1:dimIm(1)]-yOrigin)*scale;                                   %#ok<NBRAK>
        [r,c] = find(bwimg);                                                       %#ok<ASGLU>
        if ~isempty(c) 
            maxV(i) = xval(max(c));
        else
            maxV(i) = 0; 
        end
        vapPen(i) = S.S;                  % Penetration distance in mm
        tan2(i)   = theta.theta;          % Spreading angle in degrees
        [b]=jetBoundary(bwimg);             % Jet boundary
        JB{i,2}   = b(:,2)*scale+xval(1); % scaled boundary (x)
        JB{i,1}   = b(:,1)*scale+rad(1);  % scaled boundary (y)
        xind        = [xval(1) xval(end)];  % x image display range
        yind        = [rad(1) rad(end)];    % y image display range
        xt          = 10:10:100;            % Array of tick marks for x axis
        yt          = -5:5:5;               % Array of tick marks for y axis
        himage = imshow(TestImage1,CRng,'XData',xind,'Ydata',yind);            %#ok<NASGU>
        set(gca,'Visible','on','TickDir','out','XTick',xt,'Ytick',yt,...
            'XColor','k','LineWidth',1);
        xlabel('Axial distance [mm]');
        hold on; plot(JB{i,2},JB{i,1},'r','LineWidth',2); 

        % Plotting of measurements
        if ~isempty(c_L)

            % Plot "x marks the spot" if possible (estimate of penetration distance)
            %plot(round(dist),yOrigin, ...
            %        'yx', ...
            %        'MarkerSize',25, ...
            %        'LineWidth',3, ...
            %        'MarkerEdgeColor','r')

            % Plot nozzle origin (x and y are reversed due to measureSpray
            % orientation)
            x0 = xOrigin*scale + xval(1);
            y0 = yOrigin*scale + rad(1);
            plot(x0,y0,'b*')

            % Plot center of mass
            x_c = c_L.cenPos(2)*scale + xval(1);
            y_c = c_L.cenPos(1)*scale + rad(1);
            plot(x_c,y_c,'b*')

            % Plot a line through the center of mass
            xd = [x_c x0];
            yd = [y_c y0];
            xs = [min(xd) max(xd)+255];
            ys = polyval(polyfit(xd,yd,1),xs);
            plot(xs, ys, 'b--')

            % Radial distance from origin to center of mass
            r = sqrt((x_c-x0)^2 + (y_c-y0)^2);
            
            y_u = y_c - r*sind(theta.theta/2); %"left" from measureSpray is upper line here
            x_u = x0 + r*cosd(theta.theta/2);
            y_l = y_c + r*sind(theta.theta/2);
            x_l = x0 + r*cosd(theta.theta/2);
            
            % Plot upper line
            xd_u = [x0 x_l];
            yd_u = [y0 y_l];
            xs_u = [min(xd_u) max(xd_u)+250];
            ys_u = polyval(polyfit(xd_u,yd_u,1),xs_u);
            plot(xs_u,ys_u,'y--')

            % Lower line
            xd_l = [x0 x_u];
            yd_l = [y0 y_u];
            xs_l = [min(xd_l) max(xd_l)+250];
            ys_l = polyval(polyfit(xd_l,yd_l,1),xs_l);
            plot(xs_l,ys_l,'y--')

            % Plot penetration length arc
            arc_x = S.arc_y*scale + xval(1);
            arc_y = S.arc_x*scale + rad(1);
            plot(arc_x,arc_y,'b:','LineWidth',2)
            hold off
        end

        % ---- Text overlays ----
        % Frame text
        text(100,0,num2str(currentFrame), ...
            'fontsize',14, ...
            'color','g', ...
            'backgroundcolor',[0 0 0]);

        xAlign = x0-1;
        yAlign = y0+5;
        timeText = [num2str(time(i),'%4.1f') ' \mus'];
        % Time text
        if time1(i) <= endInjTime*1000
             text(xAlign,yAlign,timeText, ...
                 'fontsize',14, ...
                 'color',[1 1 1], ...
                 'backgroundcolor',[0 0 0], ...
                 'HorizontalAlignment','left', ...
                 'VerticalAlignment','top');
        else
             text(xAlign,yAlign,timeText, ...
                 'fontsize',14, ...
                 'color',[1 0 0], ...
                 'backgroundcolor',[0 0 0], ...
                 'HorizontalAlignment','left', ...
                 'VerticalAlignment','top');
        end

        % Final plotting commands
        set(gca,'Visible','on')
        set(gcf,'Color','w')
        drawnow

        fprintf('\b\b\b%3.0f',i)
    end
    fprintf('\nDone!\n\n')

    % Shifts everything by one frame for later processes (to be compatible with
    % the old JB_mm and other things)
    JB(2:end,2) = JB(1:end-1,2); JB(2:end,1) = JB(1:end-1,1); 
    JB{1,1} = []; JB{1,2} = [];
    maxV(2:end) = maxV(1:end-1); maxV(1) = 0;
    tan2(2:end) = tan2(1:end-1); tan2(1) = 0;
    time(2:end) = time(1:end-1); time(1) = 0;
    vapPen(2:end) = vapPen(1:end-1); vapPen(1) = 0;

% Spray Measurement Function
function [c_L,S,theta,A_p,theta_l] = measureSpray(img,y0,x0,pxPermm, ...
                                     siebers,avgStartPer,avgEndPer,thetaInput)
% Spray Measurements
    
% Reference:
%   Siebers, D. L., "Effects of Gas Density and Vaporization on
%   Penetration and Dispersion of Diesel Sprays," SAE Paper 960034,
%   1996.

% Centroid Position (cenPos) - Spray centroid position
% Spray Axis (c_L) - The line from the injector tip through the
%   centroid of the spray region.
% Penetration Distance (S) - The distance along the spray axis to a
%   location where 1/2 of the pixels on an arc of theta/2 centered on 
%   the spray axis are dark.
% Full Cone Angle (theta) - theta = arctan((A_p,S/2)/(S/2)^2), where
%   (A_p,S/2) is the projected spray area of the upstream half of the
%   spray in an image.
% Projected Area (A_p) - Projected area of the spray
% "Local" Spray Dispersion Angle (theta_l(x)) - theta_l =
% arctan(w(x)/x)

    if isnan(x0) || isnan(y0)
        disp('A spray origin is required to perform spray measurements...')
        c_L = []; S = []; theta = []; A_p = []; theta_l = [];
        return
    end
    if isnan(pxPermm)
        disp('A pixel per mm value is required to perform spray measurements...')
        c_L = []; S = []; theta = []; A_p = []; theta_l = [];
        return
    end
    if isnan(avgStartPer) || isnan(avgEndPer)
        disp('An average start and average end point are required to perform spray measurements...')
        c_L = []; S = []; theta = []; A_p = []; theta_l = [];
        return
    end
    if logical(~nnz(img)) %if binary image is all zeros, create small zone at nozzle so code doesn't crash
        img(x0-1:x0,y0-1:y0) = 1;
    end
    % Rotate image to make this coding simpler... it was done for a 90 degree
    % turned image.
    img = transpose(img);

    % Center Line (c_L) calculation
    %   Get the nozzle origin and centroid coordinates, output the origin,
    %   centroid, and an extrapolated line (that fits the two points). Output
    %   these values to the c_L structure.
    nozzleOrigin = [x0 y0];
    cenPos = regionprops(img,'Centroid');
        cenPos = cenPos.Centroid;
        x_c = cenPos(1);
        y_c = cenPos(2);
    xd = [x_c x0];
    yd = [y_c y0];
    c_L.xs = [min(xd) max(xd)+10];
    c_L.ys = polyval(polyfit(xd,yd,1),c_L.xs);
    c_L.origin = nozzleOrigin;
    c_L.cenPos = cenPos;
    
    % Projected area (A_p) calculation
    area = regionprops(img,'Area'); 

    % Projected area (in mm^2)
    A_p = area.Area*(1/pxPermm)^2;                                              

    % Spray angle (theta) calculation
    box = regionprops(img,'BoundingBox'); box = box.BoundingBox;
    % box(1) = upper left corner (x)
    % box(2) = upper left corner (y)
    % box(3) = x width
    % box(4) = y width

    pBox = round(box);
    % Create a vector of spray widths starting at the nozzle origin and
    % extending to the last pixel of the spray. Also calculated the total angle
    % as a function of the distance from the origin.

    % Initialize loop variables
    startPix = round(nozzleOrigin(2));
    endPix = pBox(2) + pBox(4);
    endPix(endPix>size(img,1)) = size(img,1); % Fix for occasional problems 
                                              % with '??? Index exceeds matrix 
                                              % dimensions.' errors.
    SPix = endPix - startPix;   % Length (in pixels) of the spray
    %taken outside of conditional statement so spray angle information can be
    %plotted for both tests
    w = zeros(1,SPix);
    theta_l = zeros(1,SPix);
    j = 1;
    for i = startPix:endPix
        w(j) = length(find(img(i,:)));
        theta_l(j) = 2*atand(w(j)/(2*j));
        j = j + 1;
    end
    if siebers == 1
        % Method from Siebers SAE 1999-01-0528 for spreading angle. See
        % appendix A, equation 01.
        Sover2 = round(SPix/2);
        halfSpray = img(startPix:end,:); halfSpray(Sover2:end,:)=0; % Cuts the spray in half
        area = regionprops(halfSpray,'Area');
        if length(area) > 1
            tmp = max(cell2mat(struct2cell(area)));
            clear area
            area.Area = tmp;
        end
        if length(area) < 1 || isempty(area.Area) 
            clear area
            area.Area = 0;
        end
        halfSprayArea = max(cat(1,area.Area)); % Finds maximum pixel area
        theta.theta = 2*atand(halfSprayArea/Sover2^2);
    else
        % Compute the spray angle based on an average spray angle between a
        % starting point in the spray (expressed as a percentage of the total 
        % spray length) and an ending point in the spray.
        avgStart = round(avgStartPer*SPix);
        avgEnd = round(avgEndPer*SPix);
        theta.theta = mean(theta_l(avgStart:avgEnd));
    end
    if thetaInput > 0
        theta.theta = thetaInput;
    end
    % Radial distance from origin to center of mass
    r = sqrt((x_c-x0)^2 + (y_c-y0)^2);

    % Angle offset (from the centerline)
    theta.alpha = asind((x_c-x0)/r); 

    % Points on the arc (to the left and the right)
    x_l = x_c - r*sind(theta.theta/2);
    y_l = y0 + r*cosd(theta.theta/2);
    x_r = x_c + r*sind(theta.theta/2);
    y_r = y0 + r*cosd(theta.theta/2);

    % Left hand line
    xd_l = [x_l x0];
    yd_l = [y_l y0];
    theta.xs_l = [min(xd_l)-100 max(xd_l)];
    theta.ys_l = polyval(polyfit(xd_l,yd_l,1),theta.xs_l);

    % Right hand line
    xd_r = [x_r x0];
    yd_r = [y_r y0];
    theta.xs_r = [min(xd_r) max(xd_r)+100];
    theta.ys_r = polyval(polyfit(xd_r,yd_r,1),theta.xs_r);

    % Compute the penetration length (iterative)
    onFraction = 1;
    rFrac = round(0.9*r); % Start 90% of the way to spray centroid.
    iter = 1;
     theta_S = linspace(-theta.theta/4+theta.alpha,theta.theta/4+theta.alpha,400);
 %   theta_S = linspace(-theta.theta/4,theta.theta/4,400);
    while onFraction > 0.5
        % Find an arc radius an angle of theta/2 wide near the end of the spray.
        % Discretize (by pixels) a circular path along that arc
        % Determine how many pixels along the path are on (1)
        % Divide by how many pixels are along the path.
        % If there are more pixels on than off, move one pixel outwards.
        % Continue this process until there are fewer on pixels than off pixels
        % (at least half of the pixels on the arc of theta/2 centered on the 
        % spray axis are off)

        rFrac = rFrac + 1; % Add one to the radius each iteration.

        % Points on the arc (keep this code for proper display of penetration
        % length)
        S.arc_x = (x0 + rFrac*sind(theta_S));
        S.arc_y = (y0 + rFrac*cosd(theta_S));

        % Points on the arc (rounded to the nearest pixel)
        [arcPix_x m n] = unique(round(S.arc_x)); %#ok<NASGU>
        arcPix_y = round(S.arc_y(m));

        % Catch any points < 1
        arcPix_y(arcPix_y<1) = 1;

        % Count the on pixels along the arc
        num = 0;
        for i=1:length(arcPix_x)
            imgSize = size(img);
            if arcPix_y(i)>imgSize(1)
                arcPix_y(i) = imgSize(1);
            end
            if arcPix_x(i)>imgSize(2)
                arcPix_x(i) = imgSize(2);
            end
            if arcPix_x(i)<1
                arcPix_x(i) = 1;
            end
            if arcPix_y(i)<1
                arcPix_y(i) = 1;
            end
            if img(arcPix_y(i),arcPix_x(i))==1
                num = num + 1;
            end
        end
        den = length(arcPix_x);

        onFraction = num/den;

        iter = iter + 1;
        if iter > 500 % Increase this for larger images!!!
            disp('Maximum number of iterations reached for penetration distance calculation!')
            break
        end
    end
    %plot binary image and axis to see how well this is working.
%     figure(15), clf, imshow(img)
%     hold on, plot(S.arc_x,S.arc_y,'--b')
%     plot(x0,y0,'+r')
%     plot(xd_l,yd_l,xd_r,yd_r,':')
    figure(10)
    S.SPix = rFrac;
    S.S = (rFrac)*pxPermm; % Spray penetration distance in mm.